from time import sleep

from PyQt6.QtCore import QThread, pyqtSignal, QMutex

from lanzou.api import LanZouCloud
from lanzou.api.types import RecFolder, RecFile
from lanzou.debug import logger


class GetRecListsWorker(QThread):
    '''获取回收站列表'''
    folders = pyqtSignal(object)
    infos = pyqtSignal(object, object)
    msg = pyqtSignal(str, int)

    def __init__(self, parent=None):
        super(GetRecListsWorker, self).__init__(parent)
        self._disk = None
        self._mutex = QMutex()
        self._is_work = False
        self._folder_id = None

    def set_disk(self, disk):
        self._disk = disk

    def set_values(self, fid):
        # 用于获取回收站指定文件夹内文件信息
        self._folder_id = fid
        self.start()

    def __del__(self):
        self.wait()

    def stop(self):
        self._mutex.lock()
        self._is_work = False
        self._mutex.unlock()

    def run(self):
        if not self._is_work:
            self._mutex.lock()
            self._is_work = True
            try:
                if self._folder_id:
                    file_lists = self._disk.get_rec_file_list(folder_id=self._folder_id)
                    self._folder_id = None
                    self.folders.emit(file_lists)
                    raise UserWarning
                dir_lists = self._disk.get_rec_dir_list()
                file_lists = self._disk.get_rec_file_list(folder_id=-1)
                self.infos.emit(dir_lists, file_lists)
                self.msg.emit("刷新列表成功！", 2000)
            except TimeoutError:
                self.msg.emit("网络超时，请稍后重试！", 6000)
            except UserWarning:
                pass
            except Exception as e:
                logger.error(f"GetRecListsWorker error: e={e}")
            self._is_work = False
            self._mutex.unlock()
        else:
            self.msg.emit("后台正在运行，请稍后重试！", 3100)


class RecManipulator(QThread):
    '''操作回收站'''
    msg = pyqtSignal(str, int)
    succeeded = pyqtSignal()

    def __init__(self, parent=None):
        super(RecManipulator, self).__init__(parent)
        self._disk = None
        self._mutex = QMutex()
        self._is_work = False
        self._action = None
        self._folders = []
        self._files = []

    def set_disk(self, disk):
        self._disk = disk

    def set_values(self, infos, action):
        # 操作回收站选定行
        self._action = None
        self._folders = []
        self._files = []
        for item in infos:
            if isinstance(item, RecFile):
                self._files.append(item.id)
            elif isinstance(item, RecFolder):
                self._folders.append(item.id)
        self._action = action
        self.start()

    def __del__(self):
        self.wait()

    def stop(self):
        self._mutex.lock()
        self._is_work = False
        self._mutex.unlock()

    def run(self):
        if not self._is_work:
            self._mutex.lock()
            self._is_work = True
            try:
                res = None
                if self._action == "recovery":
                    if self._files or self._folders:
                        res = self._disk.recovery_multi(self._files, self._folders)
                        if res == LanZouCloud.SUCCESS:
                            self.msg.emit("选定文件(夹)恢复成功，即将刷新列表", 2500)
                elif self._action == "delete":
                    if self._files or self._folders:
                        if self._files or self._folders:
                            res = self._disk.delete_rec_multi(self._files, self._folders)
                            if res == LanZouCloud.SUCCESS:
                                self.msg.emit("选定文件(夹)彻底删除成功，即将刷新列表", 2500)
                elif self._action == "clean":
                    res = self._disk.clean_rec()
                    if res == LanZouCloud.SUCCESS:
                        self.msg.emit("清空回收站成功，即将刷新列表", 2500)
                elif self._action == "recovery_all":
                    res = self._disk.recovery_all()
                    if res == LanZouCloud.SUCCESS:
                        self.msg.emit("文件(夹)全部还原成功，即将刷新列表", 2500)
                if isinstance(res, int):
                    if res == LanZouCloud.FAILED:
                        self.msg.emit("失败，请重试！", 4500)
                    elif res == LanZouCloud.NETWORK_ERROR:
                        self.msg.emit("网络错误，请稍后重试！", 4500)
                    else:
                        sleep(2.6)
                        self.succeeded.emit()
            except TimeoutError:
                self.msg.emit("网络超时，请稍后重试！", 6000)
            except Exception as e:
                logger.error(f"RecManipulator error: e={e}")
            self._is_work = False
            self._action = None
            self._folders = []
            self._files = []
            self._mutex.unlock()
        else:
            self.msg.emit("后台正在运行，请稍后重试！", 3100)



"""
QSS 样式
"""

from platform import system as platform

from lanzou.debug import BG_IMG

jobs_btn_redo_style = '''
QPushButton {
    color: white;
    background-color: QLinearGradient(x1: 0, y1: 0, x2: 0, y2: 1,stop: 0 #38d,
        stop: 0.1 #93e, stop: 0.49 #77c, stop: 0.5 #16b, stop: 1 #07c);
    border-width: 1px;
    border-color: #339;
    border-style: solid;
    border-radius: 7;
    padding: 3px;
    font-size: 13px;
    padding-left: 5px;
    padding-right: 5px;
    min-width: 20px;
    min-height: 14px;
    max-height: 14px;
}
'''

jobs_btn_completed_style = '''
QPushButton {
    color: white;
    background-color: QLinearGradient(x1: 0, y1: 0, x2: 0, y2: 1,stop: 0 #18d,
        stop: 0.1 #9ae, stop: 0.49 #97c, stop: 0.5 #16b, stop: 1 #77c);
    border-width: 1px;
    border-color: #339;
    border-style: solid;
    border-radius: 7;
    padding: 3px;
    font-size: 13px;
    padding-left: 5px;
    padding-right: 5px;
    min-width: 20px;
    min-height: 14px;
    max-height: 14px;
}
'''

jobs_btn_delete_style = '''
QPushButton {
    color: white;
    background-color: QLinearGradient(x1: 0, y1: 0, x2: 0, y2: 1,stop: 0 #f00,
        stop: 0.25 #008080, stop: 0.49 #97c, stop: 0.5 #16b, stop: 1 #f00);
    border-width: 1px;
    border-color: #339;
    border-style: solid;
    border-radius: 7;
    padding: 3px;
    font-size: 13px;
    padding-left: 5px;
    padding-right: 5px;
    min-width: 20px;
    min-height: 14px;
    max-height: 14px;
}
'''

jobs_btn_processing_style = '''
QPushButton {
    color: white;
    background-color: QLinearGradient(spread:pad, x1:0, y1:0, x2:1, y2:0, stop:0 rgba(9, 41, 4, 255), stop:0.085 rgba(2, 79, 0, 255), stop:0.19 rgba(50, 147, 22, 255), stop:0.275 rgba(236, 191, 49, 255), stop:0.39 rgba(243, 61, 34, 255), stop:0.555 rgba(135, 81, 60, 255), stop:0.667 rgba(121, 75, 255, 255), stop:0.825 rgba(164, 255, 244, 255), stop:0.885 rgba(104, 222, 71, 255), stop:1 rgba(93, 128, 0, 255));
    border-width: 1px;
    border-color: #339;
    border-style: solid;
    border-radius: 7;
    padding: 3px;
    font-size: 13px;
    padding-left: 5px;
    padding-right: 5px;
    min-width: 20px;
    min-height: 14px;
    max-height: 14px;
}
'''

jobs_btn_queue_style = '''
QPushButton {
    color: white;
    background-color: QLinearGradient(spread:pad, x1:0, y1:0, x2:1, y2:0, stop:0 rgba(124, 252, 0, 255), stop:0.085 rgba(186, 85, 211, 255), stop:0.19 rgba(148, 0, 211, 255), stop:0.275 rgba(255,20,147, 255), stop:0.39 rgba(112,128,144, 255), stop:0.555 rgba(112,128,144, 255), stop:0.667 rgba(255,20,147, 255), stop:0.825 rgba(148, 0, 211, 255), stop:0.885 rgba(186, 85, 211, 255), stop:1 rgba(124, 252, 0, 255));
    border-width: 1px;
    border-color: #339;
    border-style: solid;
    border-radius: 7;
    padding: 3px;
    font-size: 13px;
    padding-left: 5px;
    padding-right: 5px;
    min-width: 20px;
    min-height: 14px;
    max-height: 14px;
}
'''

qssStyle = '''
QPushButton {
    background-color: rgba(255, 130, 71, 100);
}
#table_share, #table_jobs, #table_disk, #table_rec {
    background-color: rgba(255, 255, 255, 150);
}
QTabWidget::pane {
    border: 1px;
    /* background:transparent;  # 完全透明 */
    background-color: rgba(255, 255, 255, 90);
}
QTabWidget::tab-bar {
    background:transparent;
    subcontrol-position:center;
}
QTabBar::tab {
    min-width:150px;
    min-height:30px;
    background:transparent;
}
QTabBar::tab:selected {
    color: rgb(153, 50, 204);
    background:transparent;
    font-weight:bold;
}
QTabBar::tab:!selected {
    color: rgb(28, 28, 28);
    background:transparent;
}
QTabBar::tab:hover {
    color: rgb(0, 0, 205);
    background:transparent;
}
#tabWidget QTabBar{
    background-color: #AEEEEE;
}
/*提取界面文字颜色*/
#label_share_url {
    color: rgb(255,255,60);
}
#label_dl_path {
    color: rgb(255,255,60);
}
/*网盘界面文字颜色*/
#label_disk_loc {
    color: rgb(0,0,0);
    font-weight:bold;
}
#disk_tab {
    background-color: rgba(255, 255, 255, 120);
}
/*状态栏隐藏控件分隔符*/
#statusbar {
    font-size: 14px;
    color: white;
}
#msg_label, #msg_movie_lb {
    font-size: 14px;
    color: white;
    background:transparent;
}
QStatusBar::item {
    border: None;
}
/*菜单栏*/
#menubar {
    background-color: transparent;
}
QMenuBar::item {
    color:pink;
    margin-top:4px;
    spacing: 3px;
    padding: 1px 10px;
    background: transparent;
    border-radius: 4px;
}
/* when selected using mouse or keyboard */
QMenuBar::item:selected {
    color: white;
    background: #a8a8a8;
}
QMenuBar::item:pressed {
    color: lightgreen;
    background: #888888;
}
'''

if platform() == 'Darwin':  # MacOS 不使用自定义样式
    qssStyle = ''
else:
    qssStyle = qssStyle + f"""
    #MainWindow {{
        border-image:url({BG_IMG});
    }}"""

btn_style = """
QPushButton {
    color: white;
    background-color: QLinearGradient(x1: 0, y1: 0, x2: 0, y2: 1,stop: 0 #88d,
        stop: 0.1 #99e, stop: 0.49 #77c, stop: 0.5 #66b, stop: 1 #77c);
    border-width: 1px;
    border-color: #339;
    border-style: solid;
    border-radius: 7;
    padding: 3px;
    font-size: 13px;
    padding-left: 5px;
    padding-right: 5px;
    min-width: 70px;
    min-height: 14px;
    max-height: 14px;
}
"""

others_style = """
QLabel {
    font-weight: 400;
    font-size: 14px;
}
QLineEdit {
    padding: 1px;
    border-style: solid;
    border: 2px solid gray;
    border-radius: 8px;
}
QTextEdit {
    padding: 1px;
    border-style: solid;
    border: 2px solid gray;
    border-radius: 8px;
}
#btn_chooseMultiFile, #btn_chooseDir {
    min-width: 90px;
    max-width: 90px;
}
"""

dialog_qss_style = others_style + btn_style
# https://thesmithfam.org/blog/2009/09/10/qt-stylesheets-tutorial/



from PyQt6.QtCore import QThread, pyqtSignal, QMutex

from lanzou.api import LanZouCloud
from lanzou.debug import logger
from lanzou.gui.models import Infos


class GetMoreInfoWorker(QThread):
    '''获取文件直链、文件(夹)提取码描述，用于登录后显示更多信息'''
    infos = pyqtSignal(object)
    share_url = pyqtSignal(object)
    dl_link = pyqtSignal(object)
    msg = pyqtSignal(str, int)

    def __init__(self, parent=None):
        super(GetMoreInfoWorker, self).__init__(parent)
        self._disk = None
        self._infos = None
        self._url = ''
        self._pwd = ''
        self._emit_link = False
        self._mutex = QMutex()
        self._is_work = False

    def set_disk(self, disk):
        self._disk = disk

    def set_values(self, infos, emit_link=False):
        self._infos = infos
        self._emit_link = emit_link
        self.start()

    def get_dl_link(self, url, pwd):
        self._url = url
        self._pwd = pwd
        self.start()

    def __del__(self):
        self.wait()

    def stop(self):
        self._mutex.lock()
        self._is_work = False
        self._mutex.unlock()

    def run(self):
        # infos: ID/None，文件名，大小，日期，下载次数(dl_count)，提取码(pwd)，描述(desc)，|链接(share-url)
        if not self._is_work and self._infos:
            self._mutex.lock()
            self._is_work = True
            try:
                if not self._url:  # 获取普通信息
                    if isinstance(self._infos, Infos):
                        if self._infos.id:  # 从 disk 运行
                            self.msg.emit("网络请求中，请稍候……", 0)
                            _info = self._disk.get_share_info(self._infos.id, is_file=self._infos.is_file)
                            self._infos.desc = _info.desc
                            self._infos.pwd = _info.pwd
                            self._infos.url = _info.url
                        if self._emit_link:
                            self.share_url.emit(self._infos)
                        else:
                            self.infos.emit(self._infos)
                        self.msg.emit("", 0)  # 删除提示信息
                else:  # 获取下载直链
                    res = self._disk.get_file_info_by_url(self._url, self._pwd)
                    if res.code == LanZouCloud.SUCCESS:
                        self.dl_link.emit("{}".format(res.durl or "无"))  # 下载直链
                    elif res.code == LanZouCloud.NETWORK_ERROR:
                        self.dl_link.emit("网络错误！获取失败")  # 下载直链
                    else:
                        self.dl_link.emit("其它错误！")  # 下载直链
            except TimeoutError:
                self.msg.emit("网络超时！稍后重试", 6000)
            except Exception as e:
                logger.error(f"GetMoreInfoWorker error: e={e}")
            self._is_work = False
            self._url = ''
            self._pwd = ''
            self._mutex.unlock()
        else:
            self.msg.emit("后台正在运行，请稍后重试！", 3100)



import json

import requests

from lanzou.api.utils import USER_AGENT
from lanzou.debug import logger

timeout = 2


def get_short_url(url: str):
    """短链接生成器"""
    headers = {'User-Agent': USER_AGENT}
    short_url = ""
    try:
        post_data = {"url": url}
        # 10W次/天, https证书过期
        headers["Authorization"] = "Token xxHQfao69Ra9G7EI87mC"
        resp = requests.post("https://www.dwz.lc/api/url/add", json=post_data, headers=headers, timeout=timeout)
        logger.error(resp.text)
        rsp = json.loads(resp.text)
        if rsp:
            short_url = rsp["short"]
    except Exception as e:
        logger.error(f"get_short_url error: e={e}")

    if not short_url:
        try:
            post_data = {"url": url}
            # 10W次/天, https证书过期
            headers["Authorization"] = "Token cH4lpSuC6LgqoDidiqB5"
            resp = requests.post("https://www.ecx.cx/api/url/add", json=post_data, headers=headers, timeout=timeout)
            logger.error(resp.text)
            rsp = json.loads(resp.text)
            if rsp:
                short_url = rsp["short"]
        except Exception as e:
            logger.error(f"get_short_url error: e={e}")

    if not short_url:
        try:
            # https, 国外,速度较慢
            resp = requests.get(f"https://tinyurl.com/api-create.php?url={url}", headers=headers, timeout=timeout)
            logger.error(resp.text)
            if resp.text and len(resp.text) < 32:
                short_url = resp.text
        except Exception as e:
            logger.error(f"get_short_url error: e={e}")

    if not short_url:
        headers["Content-Type"] = "application/x-www-form-urlencoded; charset=UTF-8"
        post_data = {"long_url": url}
        try:
            # 支持https, 国外,速度较慢
            resp = requests.post("http://gg.gg/create", data=post_data, headers=headers, timeout=timeout).text
            logger.error(resp)
            if resp:
                short_url = resp
        except Exception as e:
            logger.error(f"get_short_url2 error: e={e}")

    return short_url


if __name__ == "__main__":
    url = get_short_url("https://github.com/Leon406/lanzou-gui")
    print(url)



import re

from lanzou.debug import logger

"""html页面参数解析"""


def parse_file_name(html: str) -> str:
    name = re.search(r"<title>(.+?) - 蓝奏云</title>", html) or \
           re.search(r'<div class="filethetext".+?>([^<>]+?)</div>', html) or \
           re.search(r'<div style="font-size.+?>([^<>].+?)</div>', html) or \
           re.search(r"var filename = '(.+?)';", html) or \
           re.search(r'id="filenajax">(.+?)</div>', html) or \
           re.search(r'<div class="b"><span>([^<>]+?)</span></div>', html)

    return name.group(1).replace("*", "_") if name else "未匹配到文件名"


def parse_file_size(html: str) -> str:
    size = re.search(r'大小.+?(\d[\d.,]+\s?[BKM]?)<', html) or \
           re.search(r'class="n_filesize">[^<0-9]*([.0-9 MKBmkbGg]+)<', html) or \
           re.search(r'大小：(.+?)</div>', html)  # VIP 分享页面
    return size.group(1) if size else ""


def parse_time(html: str) -> str:
    time = re.search(r'class="n_file_infos">(.+?)</span>', html) or \
           re.search(r'>(\d+\s?[秒天分小][钟时]?前|[昨前]天\s?[\d:]+?|\d+\s?天前|\d{4}-\d\d-\d\d)<', html)
    return time.group(1) if time else ''


def parse_desc(html: str) -> str:
    desc = re.search(r'class="n_box_des">(.*?)</div>', html) or \
           re.search(r'文件描述.+?</span><br>\n?\s*(.*?)\s*</td>', html)
    return desc.group(1) if desc else ''


def parse_sign(html: str) -> str:
    # sign 放在变量后面前后各有一个干扰项
    sign = re.findall(r"'sign':(.+?),", html)
    logger.error("~~~~~~~~~~ first  " + "  ".join(sign))
    if len(sign) > 1:
        sign = sign[1]
    else:
        sign = sign[0]
    if len(sign) < 20:  # 此时 sign 保存在变量里面, 变量名是 sign 匹配的字符
        sign = (
                re.findall(r"var sasign\s*=\s*'(.{10,}?)';", html)
                or re.findall(r"var skdklds\s*=\s*'(.{10,}?)';", html)
                or re.findall(rf"var {sign}\s*=\s*'(.+?)';", html)
        )[-1]
    logger.error("~~~~~~~~~~ final  " + sign)
    return sign.replace("'", "")


def parse_form_hash(html: str) -> str:
    return re.findall(r'name="formhash" value="(.+?)"', html)[0]


def parse_folder_name(html: str) -> str:
    folder_name = re.search(r"var.+?='(.+?)';\n.+document.title", html) or \
                  re.search(r'user-title">(.+?)</div>', html) or \
                  re.search(r'<div class="b">(.+?)<div', html)  # 会员自定义
    return folder_name.group(1) if folder_name else ''


def parse_folder_id(html: str) -> str:
    return re.findall(r"'fid':'?(\d+)'?,", html)[0]


def parse_folder_time(html: str) -> str:
    folder_time = re.search(r'class="rets">([\d\-]+?)<a', html)  # 日期不全 %m-%d
    return folder_time.group(1) if folder_time else ''


def parse_folder_desc(html: str) -> str:
    folder_desc = re.search(r'id="filename">(.+?)</span>', html, re.DOTALL) or \
                  re.search(r'<div class="user-radio-\d"></div>(.+?)</div>', html) or \
                  re.search(r'class="teta tetb">说</span>(.+?)</div><div class="d2">', html, re.DOTALL)
    return folder_desc.group(1) if folder_desc else ''



import os

from PyQt6.QtCore import Qt, pyqtSignal
from PyQt6.QtGui import QPixmap
from PyQt6.QtWidgets import (QLineEdit, QDialog, QLabel, QDialogButtonBox, QVBoxLayout)

from lanzou.gui.qss import others_style, btn_style


class CaptchaDialog(QDialog):
    captcha = pyqtSignal(object)

    def __init__(self, parent=None):
        super(CaptchaDialog, self).__init__(parent)
        self.img_path = os.getcwd() + os.sep + 'captcha.png'
        self.initUI()
        self.setStyleSheet(others_style)

    def show_img(self):
        self.captcha_pixmap = QPixmap(self.img_path)
        self.captcha_lb.setPixmap(self.captcha_pixmap)

    def handle(self, img_data):
        with open(self.img_path, 'wb') as f:
            f.write(img_data)
            f.flush()
        self.show_img()

    def on_ok(self):
        captcha = self.code.text()
        self.captcha.emit(captcha)
        if os.path.isfile(self.img_path):
            os.remove(self.img_path)

    def initUI(self):
        self.setWindowTitle("请输入下载验证码")

        self.captcha_lb = QLabel()
        self.captcha_pixmap = QPixmap(self.img_path)
        self.captcha_lb.setPixmap(self.captcha_pixmap)

        self.code = QLineEdit()
        self.code.setPlaceholderText("在此输入验证码")

        self.buttonBox = QDialogButtonBox()
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(
            QDialogButtonBox.StandardButton.Reset | QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Close)
        self.buttonBox.button(QDialogButtonBox.StandardButton.Reset).setText("显示图片")
        self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setText("确定")
        self.buttonBox.button(QDialogButtonBox.StandardButton.Close).setText("关闭")
        self.buttonBox.setStyleSheet(btn_style)
        self.buttonBox.button(QDialogButtonBox.StandardButton.Reset).clicked.connect(self.show_img)
        self.buttonBox.accepted.connect(self.on_ok)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)

        vbox = QVBoxLayout()
        vbox.addWidget(self.captcha_lb)
        vbox.addStretch(1)
        vbox.addWidget(self.code)
        vbox.addStretch(1)
        vbox.addWidget(self.buttonBox)
        self.setLayout(vbox)
        self.setMinimumWidth(260)



from PyQt6.QtCore import Qt, pyqtSignal
from PyQt6.QtGui import QIcon
from PyQt6.QtWidgets import QDialog, QLabel, QGridLayout, QDialogButtonBox, QLineEdit

from lanzou.debug import SRC_DIR
from lanzou.gui.models import FileInfos
from lanzou.gui.qss import dialog_qss_style


class SetPwdDialog(QDialog):
    new_infos = pyqtSignal(object)

    def __init__(self, parent=None):
        super(SetPwdDialog, self).__init__(parent)
        self.infos = []
        self.initUI()
        self.update_text()
        self.setStyleSheet(dialog_qss_style)

    def set_values(self, infos):
        self.infos = infos
        self.update_text()  # 更新界面

    def set_tip(self):  # 用于提示状态
        self.setWindowTitle("请稍等……")

    def initUI(self):
        self.setWindowTitle("请稍等……")
        self.setWindowIcon(QIcon(SRC_DIR + "password.png"))
        self.lb_oldpwd = QLabel()
        self.lb_oldpwd.setText("当前提取码：")
        self.lb_oldpwd.setAlignment(
            Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignTrailing | Qt.AlignmentFlag.AlignVCenter)
        self.tx_oldpwd = QLineEdit()
        # 当前提取码 只读
        self.tx_oldpwd.setFocusPolicy(Qt.FocusPolicy.NoFocus)
        self.tx_oldpwd.setReadOnly(True)
        self.lb_newpwd = QLabel()
        self.lb_newpwd.setText("新的提取码：")
        self.lb_newpwd.setAlignment(
            Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignTrailing | Qt.AlignmentFlag.AlignVCenter)
        self.tx_newpwd = QLineEdit()

        self.buttonBox = QDialogButtonBox()
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
        self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setText("确定")
        self.buttonBox.button(QDialogButtonBox.StandardButton.Cancel).setText("取消")

        self.grid = QGridLayout()
        self.grid.setSpacing(10)
        self.grid.addWidget(self.lb_oldpwd, 1, 0)
        self.grid.addWidget(self.tx_oldpwd, 1, 1)
        self.grid.addWidget(self.lb_newpwd, 2, 0)
        self.grid.addWidget(self.tx_newpwd, 2, 1)
        self.grid.addWidget(self.buttonBox, 3, 0, 1, 2)
        self.setLayout(self.grid)
        self.buttonBox.accepted.connect(self.btn_ok)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.accepted.connect(self.set_tip)
        self.buttonBox.rejected.connect(self.reject)
        self.buttonBox.rejected.connect(self.set_tip)
        self.setMinimumWidth(280)

    def update_text(self):
        num = len(self.infos)
        if num == 1:
            self.tx_oldpwd.setVisible(True)
            self.lb_oldpwd.setVisible(True)
            infos = self.infos[0]
            if infos.has_pwd:
                self.tx_oldpwd.setText(str(infos.pwd))
                self.tx_oldpwd.setPlaceholderText("")
            else:
                self.tx_oldpwd.setText("")
                self.tx_oldpwd.setPlaceholderText("无")

            if isinstance(infos, FileInfos):  # 文件  通过size列判断是否为文件
                self.setWindowTitle("修改文件提取码")
                self.tx_newpwd.setPlaceholderText("2-6位字符,关闭请留空")
                self.tx_newpwd.setMaxLength(6)  # 最长6个字符
            else:  # 文件夹
                self.setWindowTitle("修改文件夹名提取码")
                self.tx_newpwd.setPlaceholderText("2-12位字符,关闭请留空")
                self.tx_newpwd.setMaxLength(12)  # 最长12个字符
        elif num > 1:
            self.tx_oldpwd.setVisible(False)
            self.lb_oldpwd.setVisible(False)
            self.setWindowTitle(f"批量修改{num}个文件(夹)的提取码")
            self.tx_newpwd.setPlaceholderText("2-12位字符,关闭请留空")
            self.tx_newpwd.setMaxLength(12)  # 最长12个字符
            self.tx_newpwd.setText('')
            for infos in self.infos:
                if isinstance(infos, FileInfos):  # 文件
                    self.tx_newpwd.setPlaceholderText("2-6位字符,文件无法关闭")
                    self.tx_newpwd.setMaxLength(6)  # 最长6个字符
                    break

    def btn_ok(self):
        new_pwd = self.tx_newpwd.text()
        for infos in self.infos:
            infos.new_pwd = new_pwd
        self.new_infos.emit(self.infos)  # 最后一位用于标示文件还是文件夹



from PyQt6.QtCore import Qt, pyqtSignal
from PyQt6.QtWidgets import QDialog, QLabel, QDialogButtonBox, QVBoxLayout, QPushButton, QHBoxLayout

from lanzou.gui.others import set_file_icon
from lanzou.gui.qss import others_style, btn_style


class RecFolderDialog(QDialog):
    out = pyqtSignal(object)

    def __init__(self, files, parent=None):
        super(RecFolderDialog, self).__init__(parent)
        self.files = files
        self.initUI()
        self.setStyleSheet(others_style)

    def initUI(self):
        self.setWindowTitle("查看回收站文件夹内容")
        self.form = QVBoxLayout()
        for item in iter(self.files):
            ico = QPushButton(set_file_icon(item.name), item.name)
            ico.setStyleSheet("QPushButton {border:none; background:transparent; color:black;}")
            ico.adjustSize()
            it = QLabel(f"<font color='#CCCCCC'>({item.size})</font>")
            hbox = QHBoxLayout()
            hbox.addWidget(ico)
            hbox.addStretch(1)
            hbox.addWidget(it)
            self.form.addLayout(hbox)

        self.form.setSpacing(10)
        self.buttonBox = QDialogButtonBox()
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.StandardButton.Close)
        self.buttonBox.button(QDialogButtonBox.StandardButton.Close).setText("关闭")
        self.buttonBox.setStyleSheet(btn_style)
        self.buttonBox.rejected.connect(self.reject)

        vbox = QVBoxLayout()
        vbox.addLayout(self.form)
        vbox.addStretch(1)
        vbox.addWidget(self.buttonBox)
        self.setLayout(vbox)



import os

from PyQt6.QtCore import Qt, pyqtSignal
from PyQt6.QtGui import QPixmap
from PyQt6.QtWidgets import (QDialog, QLabel, QDialogButtonBox, QLineEdit, QCheckBox,
                             QHBoxLayout, QVBoxLayout, QFormLayout, QFileDialog)

from lanzou.debug import SRC_DIR
from lanzou.gui.others import MyLineEdit, AutoResizingTextEdit
from lanzou.gui.qss import dialog_qss_style


class SettingDialog(QDialog):
    saved = pyqtSignal()

    def __init__(self, parent=None):
        super(SettingDialog, self).__init__(parent)
        self._config = object
        self.download_threads = 3
        self.max_size = 100
        self.timeout = 5
        self.dl_path = None
        self.time_fmt = False
        self.to_tray = False
        self.watch_clipboard = False
        self.debug = False
        self.set_pwd = False
        self.set_desc = False
        self.upload_delay = 0
        self.allow_big_file = False
        self.upgrade = True
        self.pwd = ""
        self.desc = ""
        self.initUI()
        self.setStyleSheet(dialog_qss_style)

    def open_dialog(self, config):
        """"打开前先更新一下显示界面"""
        self._config = config
        if self._config.name:
            self.setWindowTitle(f"设置 <{self._config.name}>")
        else:
            self.setWindowTitle("设置")
        self.cwd = self._config.path
        self.set_values()
        self.exec()

    def show_values(self):
        """控件显示值"""
        self.download_threads_var.setText(str(self.download_threads))
        self.max_size_var.setText(str(self.max_size))
        self.timeout_var.setText(str(self.timeout))
        self.dl_path_var.setText(str(self.dl_path))
        self.time_fmt_box.setChecked(self.time_fmt)
        self.to_tray_box.setChecked(self.to_tray)
        self.watch_clipboard_box.setChecked(self.watch_clipboard)
        self.debug_box.setChecked(self.debug)
        self.set_pwd_box.setChecked(self.set_pwd)
        self.set_pwd_var.setEnabled(self.set_pwd)
        self.set_pwd_var.setText(self.pwd)
        self.set_desc_box.setChecked(self.set_desc)
        self.set_desc_var.setEnabled(self.set_desc)
        self.set_desc_var.setText(self.desc)
        self.upload_delay_var.setText(str(self.upload_delay))
        self.big_file_box.setChecked(self.allow_big_file)
        self.big_file_box.setText(f"允许上传超过 {self.max_size}MB 的大文件")
        # self.big_file_box.setDisabled(True)  # 关闭允许上传大文件设置入口
        self.upgrade_box.setChecked(self.upgrade)

    def set_values(self, reset=False):
        """设置控件对应变量初始值"""
        settings = self._config.default_settings if reset else self._config.settings
        self.download_threads = settings["download_threads"]
        self.max_size = settings["max_size"]
        self.timeout = settings["timeout"]
        self.dl_path = settings["dl_path"]
        self.time_fmt = settings["time_fmt"]
        self.to_tray = settings["to_tray"]
        self.watch_clipboard = settings["watch_clipboard"]
        self.debug = settings["debug"]
        self.set_pwd = settings["set_pwd"]
        self.pwd = settings["pwd"]
        self.set_desc = settings["set_desc"]
        self.desc = settings["desc"]
        self.upload_delay = settings["upload_delay"]
        if 'upgrade' in settings:
            self.upgrade = settings["upgrade"]
        if 'allow_big_file' in settings:
            self.allow_big_file = settings["allow_big_file"]
        self.show_values()

    def get_values(self) -> dict:
        """读取输入控件的值"""
        if self.download_threads_var.text():
            self.download_threads = int(self.download_threads_var.text())
        if self.max_size_var.text():
            self.max_size = int(self.max_size_var.text())
        if self.timeout_var.text():
            self.timeout = int(self.timeout_var.text())
        if self.upload_delay_var.text():
            self.upload_delay = int(self.upload_delay_var.text())
        self.dl_path = str(self.dl_path_var.text())
        self.pwd = str(self.set_pwd_var.toPlainText())
        self.desc = str(self.set_desc_var.toPlainText())
        return {"download_threads": self.download_threads,
                "max_size": self.max_size,
                "timeout": self.timeout,
                "dl_path": self.dl_path,
                "time_fmt": self.time_fmt,
                "to_tray": self.to_tray,
                "watch_clipboard": self.watch_clipboard,
                "debug": self.debug,
                "set_pwd": self.set_pwd,
                "pwd": self.pwd,
                "set_desc": self.set_desc,
                "desc": self.desc,
                "upload_delay": self.upload_delay,
                "allow_big_file": self.allow_big_file,
                "upgrade": self.upgrade}

    def initUI(self):
        self.setWindowTitle("设置")
        logo = QLabel()
        logo.setPixmap(QPixmap(SRC_DIR + "logo2.gif"))
        logo.setStyleSheet("background-color:rgb(255,255,255);")
        logo.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.download_threads_lb = QLabel("同时下载文件数")
        self.download_threads_var = QLineEdit()
        self.download_threads_var.setPlaceholderText("范围：1-9")
        self.download_threads_var.setToolTip("范围：1-9")
        self.download_threads_var.setInputMask("D")
        self.max_size_lb = QLabel("分卷大小(MB)")
        self.max_size_var = QLineEdit()
        self.max_size_var.setPlaceholderText("普通用户最大100，vip用户根据具体情况设置")
        self.max_size_var.setToolTip("普通用户最大100，vip用户根据具体情况设置")
        self.max_size_var.setInputMask("D99")
        self.timeout_lb = QLabel("请求超时(秒)")
        self.timeout_var = QLineEdit()
        self.timeout_var.setPlaceholderText("范围：1-99")
        self.timeout_var.setToolTip("范围：1-99")
        self.timeout_var.setInputMask("D9")
        self.upload_delay_lb = QLabel("上传延时(秒)")
        self.upload_delay_var = QLineEdit()
        self.upload_delay_var.setPlaceholderText("范围：1-99")
        self.upload_delay_var.setToolTip("范围：1-99")
        self.upload_delay_var.setInputMask("D9")
        self.dl_path_lb = QLabel("下载保存路径")
        self.dl_path_var = MyLineEdit(self)
        self.dl_path_var.clicked.connect(self.set_download_path)
        self.time_fmt_box = QCheckBox("使用[年-月-日]时间格式")
        self.time_fmt_box.setToolTip("文件上传日期显示格式")
        self.to_tray_box = QCheckBox("关闭到系统托盘")
        self.to_tray_box.setToolTip("点击关闭软件按钮是最小化软件至系统托盘")
        self.watch_clipboard_box = QCheckBox("监听系统剪切板")
        self.watch_clipboard_box.setToolTip("检测到系统剪切板中有符合规范的蓝奏链接时自动唤起软件，并提取")
        self.debug_box = QCheckBox("开启调试日志")
        self.debug_box.setToolTip("记录软件 debug 信息至 debug-lanzou-gui.log 文件")
        self.set_pwd_box = QCheckBox("上传文件自动设置密码")
        self.set_pwd_var = AutoResizingTextEdit()
        self.set_pwd_var.setPlaceholderText(" 2-8 位数字或字母")
        self.set_pwd_var.setToolTip("2-8 位数字或字母")
        self.set_desc_box = QCheckBox("上传文件自动设置描述")
        self.set_desc_var = AutoResizingTextEdit()
        self.big_file_box = QCheckBox(f"允许上传超过 {self.max_size}MB 的大文件")
        self.big_file_box.setToolTip("开启大文件上传支持 (功能下线)")
        self.upgrade_box = QCheckBox("自动检测新版本")
        self.upgrade_box.setToolTip("在软件打开时自动检测是否有新的版本发布，如有则弹出更新信息")

        self.time_fmt_box.toggle()
        self.time_fmt_box.stateChanged.connect(self.change_time_fmt)
        self.to_tray_box.stateChanged.connect(self.change_to_tray)
        self.watch_clipboard_box.stateChanged.connect(self.change_watch_clipboard)
        self.debug_box.stateChanged.connect(self.change_debug)
        self.set_pwd_box.stateChanged.connect(self.change_set_pwd)
        self.set_pwd_var.editingFinished.connect(self.check_pwd)
        self.set_desc_box.stateChanged.connect(self.change_set_desc)
        self.big_file_box.stateChanged.connect(self.change_big_file)
        self.upgrade_box.stateChanged.connect(self.change_upgrade)

        buttonBox = QDialogButtonBox()
        buttonBox.setOrientation(Qt.Orientation.Horizontal)
        buttonBox.setStandardButtons(
            QDialogButtonBox.StandardButton.Reset | QDialogButtonBox.StandardButton.Save | QDialogButtonBox.StandardButton.Cancel)
        buttonBox.button(QDialogButtonBox.StandardButton.Reset).setText("重置")
        buttonBox.button(QDialogButtonBox.StandardButton.Save).setText("保存")
        buttonBox.button(QDialogButtonBox.StandardButton.Cancel).setText("取消")
        buttonBox.button(QDialogButtonBox.StandardButton.Reset).clicked.connect(lambda: self.set_values(reset=True))
        buttonBox.button(QDialogButtonBox.StandardButton.Save).clicked.connect(self.slot_save)
        buttonBox.rejected.connect(self.reject)

        form = QFormLayout()
        form.setLabelAlignment(Qt.AlignmentFlag.AlignRight)
        form.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)  # 覆盖MacOS的默认样式
        form.setSpacing(10)
        form.addRow(self.download_threads_lb, self.download_threads_var)
        form.addRow(self.timeout_lb, self.timeout_var)
        form.addRow(self.upload_delay_lb, self.upload_delay_var)
        form.addRow(self.max_size_lb, self.max_size_var)
        form.addRow(self.dl_path_lb, self.dl_path_var)

        vbox = QVBoxLayout()
        vbox.addWidget(logo)
        vbox.addStretch(1)
        vbox.addLayout(form)
        vbox.addStretch(1)
        hbox = QHBoxLayout()
        hbox.addWidget(self.time_fmt_box)
        hbox.addWidget(self.to_tray_box)
        hbox.addWidget(self.watch_clipboard_box)
        hbox.addWidget(self.debug_box)
        vbox.addLayout(hbox)
        vbox.addStretch(1)
        hbox_2 = QHBoxLayout()
        hbox_2.addWidget(self.set_pwd_box)
        hbox_2.addWidget(self.set_pwd_var)
        vbox.addLayout(hbox_2)
        vbox.addStretch(1)
        hbox_3 = QHBoxLayout()
        hbox_3.addWidget(self.set_desc_box)
        hbox_3.addWidget(self.set_desc_var)
        vbox.addLayout(hbox_3)
        hbox_4 = QHBoxLayout()
        hbox_4.addWidget(self.big_file_box)
        hbox_4.addWidget(self.upgrade_box)
        vbox.addStretch(1)
        vbox.addLayout(hbox_4)
        vbox.addStretch(2)
        vbox.addWidget(buttonBox)
        self.setLayout(vbox)
        self.setMinimumWidth(500)

    def change_time_fmt(self, state):
        if state == Qt.CheckState.Checked.value:
            self.time_fmt = True
        else:
            self.time_fmt = False

    def change_to_tray(self, state):
        if state == Qt.CheckState.Checked.value:
            self.to_tray = True
        else:
            self.to_tray = False

    def change_watch_clipboard(self, state):
        if state == Qt.CheckState.Checked.value:
            self.watch_clipboard = True
        else:
            self.watch_clipboard = False

    def change_debug(self, state):
        if state == Qt.CheckState.Checked.value:
            self.debug = True
        else:
            self.debug = False

    def change_big_file(self, state):
        if state == Qt.CheckState.Checked.value:
            self.allow_big_file = True
        else:
            self.allow_big_file = False

    def change_upgrade(self, state):
        if state == Qt.CheckState.Checked.value:
            self.upgrade = True
        else:
            self.upgrade = False

    def change_set_pwd(self, state):
        if state == Qt.CheckState.Checked.value:
            self.set_pwd = True
            self.set_pwd_var.setDisabled(False)
        else:
            self.set_pwd = False
            self.set_pwd_var.setDisabled(True)

    def change_set_desc(self, state):
        if state == Qt.CheckState.Checked.value:
            self.set_desc = True
            self.set_desc_var.setDisabled(False)
        else:
            self.set_desc = False
            self.set_desc_var.setDisabled(True)

    def check_pwd(self):
        pwd = self.set_pwd_var.toPlainText()
        pwd = ''.join(list(filter(str.isalnum, pwd)))
        if len(pwd) < 2:
            pwd = ""
        self.set_pwd_var.setText(pwd[:8])

    def set_download_path(self):
        """设置下载路径"""
        dl_path = QFileDialog.getExistingDirectory(self, "选择文件下载保存文件夹", self.cwd)
        dl_path = os.path.normpath(dl_path)  # windows backslash
        if dl_path == self.dl_path or dl_path == ".":
            return None
        self.dl_path_var.setText(dl_path)
        self.dl_path = dl_path

    def slot_save(self):
        """保存槽函数"""
        self._config.settings = self.get_values()
        self.saved.emit()
        self.close()



from random import uniform
from time import sleep, time

from PyQt6.QtCore import QThread, pyqtSignal, QMutex

from lanzou.debug import logger
from lanzou.gui.workers.down import Downloader
from lanzou.gui.workers.upload import Uploader


def change_size_unit(total):
    if total < 1 << 10:
        return "{:.2f} B".format(total)
    elif total < 1 << 20:
        return "{:.2f} KB".format(total / (1 << 10))
    elif total < 1 << 30:
        return "{:.2f} MB".format(total / (1 << 20))
    else:
        return "{:.2f} GB".format(total / (1 << 30))


def show_progress(file_name, total_size, now_size, speed=0, symbol="█"):
    """显示进度条的回调函数"""
    percent = now_size / total_size
    # 进度条长总度
    file_len = len(file_name)
    if file_len >= 20:
        bar_len = 20
    elif file_len >= 10:
        bar_len = 30
    else:
        bar_len = 40
    if total_size >= 1048576:
        unit = "MB"
        piece = 1048576
    else:
        unit = "KB"
        piece = 1024
    bar_str = ("<font color='#00CC00'>" + symbol * round(bar_len * percent)
               + "</font><font color='#000080'>" + symbol * round(bar_len * (1 - percent)) + "</font>")
    msg = "\r{:>5.1f}%\t[{}] {:.1f}/{:.1f}{} | {} | {} ".format(
        percent * 100,
        bar_str,
        now_size / piece,
        total_size / piece,
        unit,
        speed,
        file_name,
    )
    if total_size == now_size:
        msg = msg + "| <font color='blue'>Done!</font>"
    return msg


class Callback(QThread):
    '''回调显示进度'''

    def __init__(self, task, parent=None):
        super(Callback, self).__init__(parent)
        self._task = task
        self._mutex = QMutex()
        self._stopped = True
        self._emit_msg = ''

    @property
    def emit_msg(self):
        return self._emit_msg

    def run(self):
        if self._stopped:
            self._mutex.lock()
            self._stopped = False
            old_size = self._task.now_size
            old_rate = int(1000 * old_size / self._task.total_size)
            old_time = time()
            sleep(uniform(0.5, 2))
            now_size = self._task.now_size
            now_rate = int(1000 * now_size / self._task.total_size)
            now_time = time()
            if old_size != now_size and old_rate != now_rate:
                speed = change_size_unit((now_size - old_size) / (now_time - old_time)) + '/s'
                self._task.speed = speed
                self._task.rate = now_rate
                self._emit_msg = show_progress(self._task.name, self._task.total_size, self._task.now_size, speed)
            self._stopped = True
            self._mutex.unlock()


class TaskManager(QThread):
    """任务控制器线程，控制后台上传下载"""
    mgr_msg = pyqtSignal(str, int)
    mgr_finished = pyqtSignal(int)
    update = pyqtSignal()

    def __init__(self, thread=3, parent=None):
        super(TaskManager, self).__init__(parent)
        self._disk = None
        self._tasks = {}
        self._queues = []
        self._thread = thread
        self._count = 0
        self._mutex = QMutex()
        self._is_work = False
        self._old_msg = ""
        self._workers = {}
        self._allow_big_file = False

    def set_allow_big_file(self, allow_big_file):
        self._allow_big_file = allow_big_file

    def set_disk(self, disk):
        self._disk = disk

    def set_thread(self, thread):
        self._thread = thread

    def stop_task(self, task):
        """暂停任务"""
        if task.url in self._workers and self._workers[task.url].isRunning():
            logger.debug(f"Stop job: {task.url}")
            try:
                self._tasks[task.url].pause = True
                self._workers[task.url].stop()
                self._tasks[task.url].run = False
            except Exception as err:
                logger.error(f"Stop task: err={err}")
        else:
            logger.debug(f"Stop job: {task.url} is not running!")
        self.update.emit()

    def start_task(self, task):
        """开始已暂停任务"""
        if task.url not in self._workers:
            self.add_task(task)
        elif not self._workers[task.url].isRunning():
            logger.debug(f"Start job: {task}")
            self._workers[task.url].start()
            self._tasks[task.url].run = True
            self._tasks[task.url].pause = False
            self.update.emit()
        self.start()

    def add_task(self, task):
        logger.debug(f"TaskMgr add one: added={task.added}, pause={task.pause}")
        if task.url not in self._tasks.keys():
            self._tasks[task.url] = task
        task.added = False
        task.pause = False
        task.info = None
        self.start()

    def add_tasks(self, tasks: dict):
        logger.debug(f"TaskMgr add: tasks={tasks}")
        self._tasks.update(tasks)
        self.start()

    def del_task(self, task):
        logger.debug(f"TaskMgr del: url={task.url}")
        if task in self._queues:
            self._queues.remove(task)
        if task.url in self._tasks:
            del self._tasks[task.url]
        if task.url in self._workers:
            del self._workers[task.url]

    def _task_to_queue(self):
        for task in self._tasks.values():
            if not task.added and not task.pause and task not in self._queues:
                logger.debug(f"TaskMgr task2queue: url={task.url}")
                self._queues.append(task)
                task.added = True

    def __del__(self):
        self.wait()

    def _ahead_msg(self, msg):
        if self._old_msg != msg:
            if self._count == 1:
                self.mgr_msg.emit(msg, 0)
            else:
                self.mgr_msg.emit(f"有{self._count}个后台任务正在运行", 0)
            self._old_msg = msg

    def _ahead_error(self):
        self.update.emit()

    def _ahead_folder_error(self, code, file):
        # 需要单独考虑，不在 task中
        pass

    def _update_emit(self):
        self.update.emit()

    def _add_thread(self, task):
        self.update.emit()
        logger.debug(f"TaskMgr count: count={self._count}")
        self._count -= 1
        del self._workers[task.url]
        # 发送所有任务完成信号
        failed_task_num = 0
        for task in self._tasks.values():
            if not task.info:
                if task.rate < 1000:
                    return None
            else:
                failed_task_num += 1
        logger.debug(f"TaskMgr all finished!: failed_task_num={failed_task_num}")
        self.mgr_finished.emit(failed_task_num)

    def stop(self):
        self._mutex.lock()
        self._is_work = False
        self._mutex.unlock()

    def run(self):
        if not self._is_work:
            self._mutex.lock()
            self._is_work = True
            while True:
                self._task_to_queue()
                if not self._queues:
                    break
                while self._count >= self._thread:
                    self.sleep(1)
                self._count += 1
                task = self._queues.pop()
                logger.debug(f"TaskMgr run: url={task.url}")
                if task.type == 'dl':
                    self._workers[task.url] = Downloader(self._disk, task, Callback)
                    self.mgr_msg.emit(f"准备下载：<font color='#FFA500'>{task.name}</font>", 0)
                else:
                    self._workers[task.url] = Uploader(self._disk, task, Callback, self._allow_big_file)
                    self.mgr_msg.emit(f"准备上传：<font color='#FFA500'>{task.name}</font>", 0)
                try:
                    self._workers[task.url].finished_.connect(self._add_thread)
                    self._workers[task.url].proc.connect(self._ahead_msg)
                    self._workers[task.url].update.connect(self._update_emit)
                    self._workers[task.url].folder_file_failed.connect(self._ahead_folder_error)
                    self._workers[task.url].failed.connect(self._ahead_error)
                    self._workers[task.url].start()
                except Exception as err:
                    logger.error(f"TaskMgr Error: err={err}")
            self._is_work = False
            self._mutex.unlock()



from PyQt6.QtCore import Qt, pyqtSignal
from PyQt6.QtGui import QIcon, QStandardItem, QStandardItemModel
from PyQt6.QtWidgets import QDialog, QLabel, QListView, QDialogButtonBox, QVBoxLayout

from lanzou.debug import SRC_DIR
from lanzou.gui.others import set_file_icon
from lanzou.gui.qss import dialog_qss_style


class DeleteDialog(QDialog):
    new_infos = pyqtSignal(object)

    def __init__(self, infos, parent=None):
        super(DeleteDialog, self).__init__(parent)
        self.infos = infos
        self.out = []
        self.initUI()
        self.setStyleSheet(dialog_qss_style)

    def initUI(self):
        self.setWindowTitle("确认删除")
        self.setWindowIcon(QIcon(SRC_DIR + "delete.png"))
        self.layout = QVBoxLayout()
        self.list_view = QListView()
        self.list_view.setViewMode(QListView.ViewMode.ListMode)
        # 列表
        self.slm = QStandardItem()
        self.model = QStandardItemModel()
        max_len = 10
        count = 0
        for info in self.infos:
            if info.is_file:  # 文件
                self.model.appendRow(QStandardItem(set_file_icon(info.name), info.name))
            else:
                self.model.appendRow(QStandardItem(QIcon(SRC_DIR + "folder.gif"), info.name))
            self.out.append({'fid': info.id, 'is_file': info.is_file, 'name': info.name})  # id，文件标示, 文件名
            count += 1
            if max_len < len(info.name):  # 使用最大文件名长度
                max_len = len(info.name)
        self.list_view.setModel(self.model)

        self.lb_name = QLabel("尝试删除以下{}个文件(夹)：".format(count))
        self.buttonBox = QDialogButtonBox()
        self.buttonBox.setOrientation(Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel)
        self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setText("确定")
        self.buttonBox.button(QDialogButtonBox.StandardButton.Cancel).setText("取消")

        self.layout.addWidget(self.lb_name)
        self.layout.addWidget(self.list_view)
        self.layout.addWidget(self.buttonBox)
        self.setLayout(self.layout)

        self.buttonBox.accepted.connect(self.btn_ok)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)
        self.setMinimumWidth(400)
        self.resize(int(max_len * 8), int(count * 34 + 60))

    def btn_ok(self):
        self.new_infos.emit(self.out)
